@@ -17,7 +17,7 @@ from inspect import getmro
 from unittest import expectedFailure
 from unittest import TestCase as StdlibTestCase
 
-from twisted.python import log
+from twisted.python import log, reflect
 from twisted.python.failure import Failure
 from twisted.trial import itrial, unittest, runner, reporter, util
 from twisted.trial.reporter import _ExitWrapper, UncleanWarningsReporterWrapper
@@ -206,6 +206,12 @@ class ErrorReportingTests(StringTest):
         use iterate() directly. See
         L{erroneous.DelayedCall.testHiddenException} for more details.
         """
+        from twisted.internet import reactor
+        if reflect.qual(reactor).startswith("twisted.internet.asyncioreactor"):
+            raise self.skipTest(
+                ("This test does not work on the asyncio reactor, as the "
+                 "traceback comes from inside asyncio, not Twisted."))
+
         test = erroneous.DelayedCall('testHiddenException')
         output = self.getOutput(test).splitlines()
         if _PY3:
